function runRSVP(Randomize);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%runRSVP(RSVPname,lag,randomize);
%
%Runs a series of RSVP streams through the model.
%This is where all of the work is done
%
%Inputs:
%
%RSVPname:  the name of the RSVP stream, generated in inputpatterns.m
%
%lag:  which lag(s) to one.  
%If lag = 0, the model will run lags 1 through NUMSTREAMS
%If lag > 0, the model will run only that lag
%
%randomize:  determines if random variance will be added to input patterns,
%passed directly to generateinput
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

global BIAS INHIB LEAK DT_VM EE EI EL GAMMA THETA TIMESTEP NUMLAYERS NUMNEURONS NUMLAGS LAGLENGTH ITEMLENGTH NUMWM TDU TDUWEIGHT STARTVAL TEMPDELTATHRESH KNOCKOUTTHRESH PERMTHRESH TEMPRATE PERMRATE NUMSTREAMS NUMITEMS RESLIMIT VARIANCE BASE RESWEIGHT NUMINPUTS THETA2 WIPELAYER
global Neurons Axons  Inputvector  Retina1 Retina2 Weightcount Weightlist Axonindex LearnItemDelta LearnItem 
global History MPHistory OutHistory InHistory BiasHistory ExHistory  TempHistory  TempdeltaHistory  PermdeltaHistory PermHistory  HebbHistory Energy Eout EnergyHistory
global BASICRSVP WMBIAS NUMNEURONSBYLAYER  CONFLICT NUMSTEPS REALSTARTVAL Streamanswers BIASVARIANCE
global Weightparams negbias Carteractivation colornaming GAMMA2

%trial bbased globals
global Trialnum Lasttrialnum Thisvariance Inputstream Stepnumber


%temporary globals here
global step sigout bias hebbvals


architecture;  %function to initialize network parameters everything
%inputpatterns

%calculate number of steps to run the model for.  
stepperlag = LAGLENGTH/TIMESTEP;
totalsteps = stepperlag * (NUMSTEPS);
totalsteps = 1000;


runlength = totalsteps;

newneurons = zeros(NUMLAYERS,NUMINPUTS);


%the Histories are used for analyzing the network
hebbvals = zeros(NUMLAYERS,NUMLAYERS,NUMINPUTS,NUMINPUTS);
HebbHistory = zeros(NUMSTREAMS,runlength,NUMINPUTS,NUMINPUTS);      %this is the one actually used by the figure scripts
History = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);      %this is the one actually used by the figure scripts
OutHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);   %sigout, output of neurons
MPHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);     %membrane potential
ExHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);     %excitatory force
BiasHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);   %Bias
EnergyHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS);   %Bias

InHistory = zeros(NUMSTREAMS,runlength,NUMLAYERS,NUMINPUTS);                %inhibition history
Energy = zeros(NUMLAYERS,1);


layerbias = zeros(1,NUMLAYERS);

sigout = zeros(NUMLAYERS,NUMINPUTS);  %output of neurons
sigout2 = zeros(NUMLAYERS,NUMINPUTS);  %output of neurons


Trialnum = 1;
totalsteps = 1;
Lasttrialnum = 0;


%This is where it begins, the main loop.  

Neurons = zeros(NUMLAYERS,NUMNEURONS);
architecture;  %function to initialize network parameters everything


%reversearchitecture;    %for algom
%dmclarchitecture;    % for dmcl


for(i= 1:NUMLAYERS)
    Neurons(i,:) = STARTVAL(i);
end
NUMSTREAMS = 10;
while(Trialnum <= NUMSTREAMS & totalsteps < 11500)
    
    Trialnum;
    %from the appropriate element of RSVPname, generate the actual input stream at TIMESTEP resolution    
    if(Trialnum ~=Lasttrialnum)  %initialize the Neurons to STARTVAL unless WIPELAYER says otherwise.  
        %do new trialstuff here
        step = 1;
    else 
        step = step + 1;    
    end
    totalsteps = totalsteps + 1;
    
    generatenextinput(Randomize);
    
    
    %**********************************************************************
    %this is the main for-loop for an RSVP presentation
    
    for(lay=3:NUMLAYERS)  %precompute outputs to speed up processing
        for(n = 1:NUMNEURONSBYLAYER(lay))
            out = Neurons(lay,n)-THETA(lay);
            out2 = Neurons(lay,n)-THETA2(lay);
            %       sigout(lay,n) = (out*GAMMA(lay))/(out*GAMMA(lay)+1);    %our cheap sigmoid, THETA is threshold, GAMMA controls shape
            
            
            sigout(lay,n) = 1/(1+exp(-GAMMA(lay)*(out)));   %real sigmoid                
            sigout2(lay,n) = 1/(1+exp(-GAMMA2(lay)*(out2)));

            %                sigout2(lay,n) = 1/(1+exp(-out2));   %real sigmoid                
        end
    end
    
    testthresh = -.95;
    for(layer = 11:11);
        
        %calculate energy here
        Energy(layer) = 0;
        
        for(neurona = 1:NUMNEURONSBYLAYER(layer))   
            for(neuronb = neurona+1:NUMNEURONSBYLAYER(layer))   
                Energy(layer) = Energy(layer) + max(0,Neurons(layer,neurona)-testthresh)*max(0,Neurons(layer,neuronb)-testthresh);
            end
        end
        Energy(layer) = Energy(layer)/  NUMNEURONSBYLAYER(layer)   ;
        Eout(layer) = max(0,1/(1+exp(-Energy(layer)))-.5);  %apply a crude sigmoid
        
        if(Eout(10) < .01)
            Eout(10) = 0;
        end
    end 
    
    
    %copy the input patterns in to the retina  (layers 1 and 2 in this case)    
    
    Neurons(1,1:NUMINPUTS) = Retina1(1,1:NUMINPUTS);  %copy this step's input into V1    
    sigout(1,1:NUMINPUTS)= Retina1(1,1:NUMINPUTS);
    Neurons(2,1:NUMINPUTS) = Retina2(1,1:NUMINPUTS);  %copy this step's input into V1    
    sigout(2,1:NUMINPUTS)= Retina2(1,1:NUMINPUTS);
    
    bias = zeros(NUMLAYERS,NUMINPUTS);
    negbias = zeros(NUMLAYERS,NUMINPUTS);
    layerbias = zeros(1,NUMLAYERS);
    
    %weights are done via lists and indexes rather than exhaustive
    %arrays, which are inefficient for a sparse matrix.
    
    
    for(w = 1:Weightcount);    %run through the list of weights and compile them for normal connections                       
        ww = Weightparams(w,5);
        wt = Weightparams(w,6);
        sat = Weightparams(w,7);
        stp = Weightparams(w,8);
        whichthresh = Weightparams(w,14);
        if(wt ==0)            %weight type
            layer1 = Weightparams(w,1);
            layer2 = Weightparams(w,2);
            n1 = Weightparams(w,3);
            n2 = Weightparams(w,4);               
            if(whichthresh ==1)
                so = sigout(layer1,n1);
            else  %use a threshold of .5 if weightthesh = 2
                so = sigout2(layer1,n1);
            end
            
            
            if(ww > 0)
                bias(layer2,n2) = bias(layer2,n2) + min(sat,so)*ww *stp;  %normalize by num neurons?  
            else
                negbias(layer2,n2) = negbias(layer2,n2) + min(sat,so)*ww;  %normalize by num neurons?     
            end
        end            
        
        %compute synaptic input that is not specific
        if(wt == 1)     
            layer1 = Weightparams(w,1);
            layer2 = Weightparams(w,2);
            n1 = Weightparams(w,3);
            n2 = Weightparams(w,4);
            layerbias(layer2) =layerbias(layer2) +sigout(layer1,n1)*ww;
        end
    end
    
    %add the conflict energy 
    bias(10,:) = bias(10,:) + Eout(11)*CONFLICT;
    
    for(layer = 3:NUMLAYERS)   
        
        
        if(BIAS(layer) > 0)
            bias(layer,:) = bias(layer,:) + BIAS(layer);
        else
            negbias(layer,:)=   negbias(layer,:) +BIAS(layer);
        end
        MINS(1:13) = -5;
        if(colornaming)
         %   MINS(5) = -.065;
        end
        
        for(neuron = 1:NUMNEURONSBYLAYER(layer))   %for each neuron in this layer                               
            ExHistory(Trialnum,step,layer,neuron) = negbias(layer,neuron);    
            change = bias(layer,neuron) + negbias(layer,neuron);  
            newneurons(layer,neuron) = max(MINS(layer),  (1-DT_VM(layer))*Neurons(layer,neuron) + DT_VM(layer)*change);
            
        end  %for neuron
    end % for layer
    
    
    %finish recording the data for this step.
    HebbHistory(Trialnum,step,:,:) = reshape(hebbvals(8,4,:,:),[1,1,NUMINPUTS,NUMINPUTS]);
    BiasHistory(Trialnum,step,:,:) = reshape(bias,[1,1,NUMLAYERS,NUMINPUTS]);
    OutHistory(Trialnum,step,:,:) = reshape(sigout,[1,1,NUMLAYERS,NUMINPUTS]);
    InHistory(Trialnum,step,:,:) = reshape(sigout2,[1,1,NUMLAYERS,NUMINPUTS]);
    MPHistory(Trialnum,step,:,:) = reshape(Neurons,[1,1,NUMLAYERS,NUMINPUTS]);
    EnergyHistory(Trialnum,step,:) = reshape(Energy,[1,1,NUMLAYERS]);
    
    
    %are we at a new trial?
    Lasttrialnum = Trialnum;
    if(step > 600)
        responseratio1 = sigout(7,1)/sigout(7,2);
        responseratio2 = sigout(7,2)/sigout(7,1);
        
        %when did the response occur on this trial?  We have to
        %do this here to reset values properly
        len = size(HebbHistory,2);
        Thresh = 0;
        checkuntil = 600;
        thislag  = Trialnum;
        answer = Streamanswers(thislag);
        nonanswer1 = 3-Streamanswers(thislag);
        nonanswer2 = 3;
        MP1 =  MPHistory(thislag,1:checkuntil,11,answer) > Thresh;
        MP2 =  MPHistory(thislag,1:checkuntil,11,nonanswer1) > Thresh;
        MP3 =  MPHistory(thislag,1:checkuntil,11,nonanswer2) > Thresh;
        
            
     [x1 ind1] = max(MP1);
     [x2 ind2] = max(MP2);
     
     RT = 0;
     if(x1 == 1 & x2 ==0)
         RT = ind1;
     end
     if(x1 == 0 & x2 ==1)
         RT = ind2*-1;
     end
     if(x1 == 1 & x2 ==1)
         
         [RT which] = min([ind1,ind2]);
         if(which ==2)
             RT = RT* -1;
         end
     end
     
        
        %change this bit if you want trials to stop/start immediately.                    
        if(RT == 0)
            tp = checkuntil;
        else
            tp = abs(RT)+100;
            tp = checkuntil;  %delete this to have trials start immediately
        end
        
        for(lay = 1: NUMLAYERS)
            
            if(WIPELAYER(Trialnum,lay)==0)
                REALSTARTVAL(lay) = MPHistory(thislag,tp,lay,1);
            end
        end
        
        
        for(lay= 1:NUMLAYERS)
            
            %                  if(WIPELAYER(Trialnum,lay))
            newneurons(lay,:) = REALSTARTVAL(lay);
            %                  end
            
        end
        Carteractivation(Trialnum) =  mean(max(MPHistory(Trialnum,1:checkuntil,10,1) - .65,0));
        
        Trialnum = Trialnum + 1;                
    end
    %set the membrane potentials to equal the new ones
    Neurons(1:NUMLAYERS,1:NUMINPUTS) = newneurons;
    
end %while


History = MPHistory;
History(:,1,:,1) = 1;

%evaluate the performance of the model
evalperformance

return




